import math
from pyglet.window import key

from pyglet.gl import *

import cocos
import cocos.euclid as eu
from cocos.director import director
from cocos.actions import Reverse, MoveTo, Place, Repeat
from cocos import path as cpa


from svgnode import SVGNode
import sprite
import squirtle

from data import filepath
from whitei import Whitei
import evil_creatures


def direction(v):
    return -math.atan2(v.y, v.x) * 180 / math.pi

class ActualLevel(cocos.layer.ScrollableLayer):
    is_event_handler = True
    def __init__(self, level_number):
        self.level_number = level_number
        super(ActualLevel,self).__init__()

        self.level_layers = squirtle.load_layers(filepath('level_%d.svg' % self.level_number),
                anchor_x = 'left',
                anchor_y = 'bottom',
                )
        temp = self.level_layers['background']
        
        self.px_height = temp.height
        self.px_width = temp.width

        self.svg = SVGNode(temp)
        self.add(self.svg)
        self.svg.position = 0, 0 # self.px_width/2, self.px_height/2

        self.player = Whitei()
        self.player.position = 20, 100

        self.add(self.player)

        temp = self.level_layers['foreground']

        svg = SVGNode(temp)
        self.add(svg,z=1000)
        svg.position = self.px_width/2, self.px_height/2
        svg.position = 0, 0 

        self.bindings = { #key constant : button name
            key.LEFT:'left',
            key.RIGHT:'right',
            key.UP:'up',
            key.DOWN:'down',
            key.PLUS:'zoomin',
            key.MINUS:'zoomout'
            }
        self.buttons = { #button name : current value, 0 not pressed, 1 pressed
                'left':0,
                'right':0,
                'up':0,
                'down':0,
                'zoomin':0,
                'zoomout':0
                }

        self.score = 0
        self.tines_collected = 0


        self.setup_allowed_area()

        self.load_sprites()

        self.schedule(self.step)

        

    def load_sprites(self):
        self.sprites = []
        sprites_layer  = squirtle.load_layers(filepath('sprite.svg' ),
                anchor_x = 'left',
                anchor_y = 'bottom',
                )
        temp = sprites_layer['Layer 1']

        sprite_locations_layer = self.level_layers['sprite_locations']


        for name in ('clam', 'tine_silver', 'star'):
            if name in sprite_locations_layer.paths_by_label:
                for location in sprite_locations_layer.paths_by_label[name]:
                    cls_name = name[0].upper() + name[1:]
                    if hasattr(sprite, cls_name):
                        cls = getattr(sprite, cls_name)
                    else:
                        cls = sprite.SpriteFromPaths
                    tmp = cls(temp.paths_by_label[name])

                    self.add(tmp, z = 10)
                    tmp.position = location.get_box().center
                    self.sprites.append(tmp)

        self.need_updateing = []
        def load_evil_sprite(name):
            if name in sprite_locations_layer.paths_by_label:
                path = sprite_locations_layer.paths_by_label[name]

                cls_name = name[0].upper() + name[1:]
                cls = getattr(evil_creatures, cls_name)

                es  = cls(sprite_locations_layer.paths_by_label[name][0].get_box())
                self.need_updateing.append(es)
                self.add(es, z=100)

                np =[]

                for point in path[0].path[0]:
                    new_point = (point[0], 700 - point[1])
                    np.append(new_point)

                trying_to_work_out = Place(np[0])
                for point in np[1:]:
                    trying_to_work_out += MoveTo(point, es.fastness)
                np.reverse()
                for point in np:
                    trying_to_work_out += MoveTo(point, es.fastness)

                es.do(Repeat(trying_to_work_out))

        for name in ('crab', 'fish', 'octopus'):
            load_evil_sprite(name)

        
    def setup_allowed_area(self):
        self.allowed_rects=[]
        allowed_level = self.level_layers['allowed_area']
        for path in allowed_level.paths:
            self.allowed_rects.append(path.get_box())


    def on_key_press(self, k, m ):
        binds = self.bindings
        if k in binds:
            self.buttons[binds[k]] = 1
            return True
        return False

    def on_key_release(self, k, m ):
        binds = self.bindings
        if k in binds:
            self.buttons[binds[k]] = 0
            return True
        return False

    def on_enter(self):
        super(ActualLevel,self).on_enter()
        self.scroller = self.get_ancestor(cocos.layer.ScrollingManager)
        self.scene = self.get_ancestor(cocos.scene.Scene)
        self.scroller.set_focus(*self.player.position)
        self.scene.set_score(self.score)
        self.scene.set_health(self.player.health)
        self.scene.set_tines(self.tines_collected)

    def step( self, dt ):
        buttons = self.buttons
        move_dir = eu.Vector2(buttons['right']-buttons['left'],
                              buttons['up']-buttons['down'])
        if move_dir:
            new_pos = self.player.position + self.player.fastness*dt*move_dir.normalize()
            for allowed_rect in self.allowed_rects:
                if allowed_rect.contains(*new_pos):
                    break
            else:
                new_pos = self.player.position

            self.player.position = new_pos
            self.player.direction= direction(move_dir) 

            self.scroller.set_focus(*self.player.position)
            for sprite in self.sprites:
                if sprite.contains(*new_pos):
                    self.sprites.remove(sprite)
                    self.remove(sprite)
                    self.score += sprite.points
                    self.scene.set_score(self.score)
                    if sprite.is_tine:
                        self.tines_collected += 1
                        self.scene.set_tines(self.tines_collected)
                    
                    

        for nu in self.need_updateing:
            if nu.contains(*self.player.position):
                self.player.health -= 1
                self.scene.set_health(self.player.health)

        if self.player.health < 1:
            print 'game over, loser'
        elif self.tines_collected == 3:
            self.scene.display_level_finished()
            load_level(self.level_number + 1)()



class Level(cocos.scene.Scene):
    def __init__(self, level_number):
        self.level_number = level_number
        super(Level, self).__init__()
        world_layer = ActualLevel(level_number)
        scroller = cocos.layer.ScrollingManager()
        scroller.add(world_layer)
        self.add(scroller)
        self.scroller = scroller
        font_name = 'BirthdayGreetz'
        self.score = cocos.text.Label('0',
                font_name= font_name,
                font_size=18,
                anchor_x='left', anchor_y='top')
        self.add(self.score, z=1)
        self.score.position  = 10, director.get_window_size()[1] - 10

        self.health = cocos.text.Label('0',
                font_name= font_name,
                font_size=18,
                anchor_x='left', anchor_y='top')
        self.add(self.health, z=1)
        self.health.position  = 10, director.get_window_size()[1] - 35

        self.tines = cocos.text.Label('0',
                font_name= font_name,
                font_size=18,
                anchor_x='left', anchor_y='top')
        self.add(self.tines, z=1)
        self.tines.position  = 10, director.get_window_size()[1] - 60

    def set_score(self, value):
        self.score.element.text = 'Score: %d' % value

    def set_health(self, value):
        self.health.element.text = 'Heath: %d' % value

    def set_tines(self, value):
        self.tines.element.text = 'Tines Collected: %d' % value

    def display_level_finished(self):
        t  = cocos.text.Label('Level Complete',
                font_name= 'BirthdayGreetz',
                font_size=18,
                anchor_x='center', anchor_y='center')
        self.add(t, z=100)
        t.position = director.get_window_size()[0]/2, director.get_window_size()[1]/2
        


class LoadingScene(cocos.scene.Scene):
    def __init__(self, level_number, message):
        super(LoadingScene,self).__init__()
        label = cocos.text.Label(message,
                font_name= 'BirthdayGreetz',
                font_size=18,
                anchor_x='center', anchor_y='center')
        self.add(label)
        label.position  = director.get_window_size()[0]/2, director.get_window_size()[1]/2
        self.level_number = level_number
        self.schedule(self.step)

    def step( self, dt ):
        l = Level(self.level_number)
        director.replace(l)

def load_level(level_number):
    def load():
        s = LoadingScene(level_number, "Loading....")
        director.replace(s)

    return load
